/**
 * Copyright 2006 Ramona Dromereschi
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/
package com.puzzle;

import com.intellij.openapi.components.ProjectComponent;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.wm.ToolWindow;
import com.intellij.openapi.wm.ToolWindowAnchor;
import com.intellij.openapi.wm.ToolWindowManager;
import com.intellij.util.ui.UIUtil;
import com.puzzle.game.NewGameListener;
import com.puzzle.game.impl.PuzzleController;
import com.puzzle.game.impl.PuzzleModel;
import com.puzzle.settings.SettingsPanel;
import com.puzzle.settings.StartGameListener;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import java.awt.BorderLayout;

/**
 * Created by IntelliJ IDEA.
 * <p/>
 * User: Ramona Dromereschi
 * <p/>
 * Date: Oct 25, 2006
 * Time: 1:30:20 PM
 * <p/>
 * The main plugin class.
 */
public class PuzzleTool implements ProjectComponent, NewGameListener, StartGameListener {
    /** The tool window ID. */
    private static final String TOOL_WINDOW_ID = "Puzzle";
    /** The application main panel. */
    private JPanel mainPanel;
    /** The project reference. */
    private Project mainProject;
    /** The plugin image icon. */
    private ImageIcon image = new ImageIcon(getClass().getResource("images/puzzle.png"));
    /** The settings panel. */
    private SettingsPanel panel = new SettingsPanel(image);
    /** The puzzle controller. */
    private PuzzleController controller;

    /**
     * The main class constructor.
     * @param project the current project
     */
    public PuzzleTool(Project project) {
        mainProject = project;
    }

    /** Initializes the component. */
    public void initComponent() {
    }

    /** Disposes the component. */
    public void disposeComponent() {
    }

    /**
     * Returns the component name.
     * @return the component name
     */
    public String getComponentName() {
        return "PuzzleTool";
    }

    /** The actions to be executed when the project is opened. */
    public void projectOpened() {
        initToolWindow();
    }

    /** The actions to be executed when the project is closed. */
    public void projectClosed() {
        unregisterToolWindow();
    }


    /** Initializes the tool window. */
    private void initToolWindow() {
        ToolWindowManager toolWindowManager = ToolWindowManager.getInstance(mainProject);
        panel.addStartGameListener(this);
        mainPanel = new JPanel(new BorderLayout());
        mainPanel.setBackground(UIUtil.getTreeTextBackground());
        mainPanel.add(panel, BorderLayout.WEST);
        initGame(0);
        ToolWindow toolWindow = toolWindowManager.registerToolWindow(TOOL_WINDOW_ID, mainPanel, ToolWindowAnchor.BOTTOM);
        toolWindow.setIcon(image);
        toolWindow.setTitle("Puzzle");
    }

    /** The actions to be executed when unregistering the window. */
    private void unregisterToolWindow() {
        ToolWindowManager toolWindowManager = ToolWindowManager.getInstance(mainProject);
        toolWindowManager.unregisterToolWindow(TOOL_WINDOW_ID);
    }

    /**
     * Received a new game request notification.
     */
    public void newGameRequest() {
        mainPanel.add(panel, BorderLayout.WEST);
        mainPanel.updateUI();
        panel.getComponent(0).requestFocus();
    }

    /**
     * Received a start new game notification.
     * @param level the game level
     */
    public void startNewGame(int level) {
        mainPanel.removeAll();
        initGame(level + 2);
        mainPanel.updateUI();
        mainPanel.repaint();
        mainPanel.requestFocus();
    }

    /**
     * Initializes a new game.
     * @param level the game level
     */
    private void initGame(int level) {
        PuzzleModel model = new PuzzleModel(level);
        if (controller != null) {
            controller.getView().removeNewGameListener(this);
        }
        controller = new PuzzleController(mainPanel, model);
        mainPanel.add(controller.getVisualComponent(), BorderLayout.CENTER);
        controller.getView().addNewGameListener(this);
    }
}
