/**
 * Copyright 2006 Ramona Dromereschi
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/
package com.puzzle.game.impl;

import com.puzzle.game.states.IMoveState;
import com.puzzle.game.states.impl.DownState;
import com.puzzle.game.states.impl.UpState;
import com.puzzle.game.states.impl.LeftState;
import com.puzzle.game.states.impl.RightState;

import java.awt.Component;
import java.awt.event.KeyEvent;
import java.util.Map;
import java.util.HashMap;
import java.util.Random;

/**
 * Created by IntelliJ IDEA.
 * <p/>
 * User: Ramona Dromereschi
 * <p/>
 * Date: Oct 25, 2006
 * Time: 2:24:56 PM
 * <p/>
 * The puzzle controller.
 */
public class PuzzleController {
    /** The puzzle model. */
    private PuzzleModel model;
    /** The puzzle view. */
    private PuzzleView view;
    /** A map between key codes and move states. */
    private Map<Integer, IMoveState> keyCodeToState = new HashMap<Integer, IMoveState>();

    /**
     * The class constructor.
     * @param p the parent component
     * @param m the puzzle model
     */
    public PuzzleController(Component p, PuzzleModel m) {
        model = m;
        view = new PuzzleView(this, model);
        p.addKeyListener(view);
        initStates();
    }

    /**
     * Updates the view size.
     */
    public void updateSize() {
        model.setPuzzleY((view.getWidth() - model.getMaxWidth() * model.getPieceSize()) / 2);
        model.setPuzzleX((view.getHeight() - model.getStatusBarHeight()
                - model.getMaxHeight() * model.getPieceSize()) / 2);
    }

    /**
     * Updates the puzzle status, based on the key pressed.
     * @param keyCode the key code
     */
    public void updatePuzzle(int keyCode) {
        IMoveState currentState = keyCodeToState.get(keyCode);
        if (currentState != null) {
            currentState.move();
        }
    }

    /**
     * Checks if the current puzzle status is a winning one.
     * @return <code>true</code> if the current puzzle status is a winning one
     */
    public boolean isWinner() {
        model.setWinner(true);
        int i = 0, j;
        while ((i < model.getMaxHeight()) && (model.isWinner())) {
            j = 0;
            while ((j < model.getMaxWidth()) && (model.isWinner())) {
                if (model.getMatPuzzle()[i][j] != 0) {
                    int val = i * model.getMaxHeight() + (j + 1);
                    if (model.getMatPuzzle()[i][j] != val) {
                        model.setWinner(false);
                    }
                }
                j++;
            }
            i++;
        }
        return model.isWinner();
    }

    /**
     * Returns the puzzle visual component.
     * @return the puzzle visual component
     */
    public Component getVisualComponent() {
        return view;
    }

    /**
     * Returns the puzzle view.
     * @return the puzzle view
     */
    public PuzzleView getView() {
        return view;
    }

    /**
     * Initializes the states mapping.
     */
    private void initStates() {
        keyCodeToState.put(KeyEvent.VK_DOWN, new DownState(view, model));
        keyCodeToState.put(KeyEvent.VK_UP, new UpState(view, model));
        keyCodeToState.put(KeyEvent.VK_LEFT, new LeftState(view, model));
        keyCodeToState.put(KeyEvent.VK_RIGHT, new RightState(view, model));
    }

    /**
     * Generates and returns a random number.
     * @return a random number
     */
    public int getRandomNumber() {
        int randomNr = Math.abs(new Random().nextInt() % (model.getCountNumbers() - 1));
        int number = model.getNumbers()[randomNr];
        for (int i = randomNr; i < model.getCountNumbers() - 1; i++) {
            model.getNumbers()[i] = model.getNumbers()[i + 1];
        }
        model.setCountNumbers(model.getCountNumbers() - 1);
        return number;
    }



}
