/**
 * Copyright 2006 Ramona Dromereschi
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/
package com.puzzle.game.impl;

/**
 * Created by IntelliJ IDEA.
 * <p/>
 * User: Ramona Dromereschi
 * <p/>
 * Date: Oct 25, 2006
 * Time: 2:16:47 PM
 * <p/>
 * The puzzle model.
 */
public class PuzzleModel {
    /** The status bar height. */
    private int statusBarHeight = 20;
    /** The current puzzle values. */
    private int[][] matPuzzle;
    /** All the existing number values for puzzle. **/
    private int[] numbers;
    /** The number of the used numbers for puzzle. **/
    private int countNumbers = 0;
    /** The current number of the seconds of the puzzle game timer. **/
    private int currentSeconds = 0;
    /** The current number of the minutes of the puzzle game timer. **/
    private int currentMinutes = 0;
    /** The current number of the total moves done by the player on the puzzle game. **/
    private int countMoves = 0;
    /** The standard puzzle game piece size. **/
    private int pieceSize = 20;
    /** The maximum allowed width of the puzzle game drawing. **/
    private int maxWidth;
    /** The maximum allowed height of the puzzle game drawing. **/
    private int maxHeight;
    /** The X current position in the puzzle game. **/
    private int currentX = 0;
    /** The Y current position in the puzzle game. **/
    private int currentY = 0;
    /** The start X position of drawing area for the puzzle table. **/
    private int puzzleX = 0;
    /** The start Y position of drawing area for the puzzle table. **/
    private int puzzleY = 0;
    /** A flag to be <code>true</code> if the current game has a winner. **/
    private boolean winner = false;

    /**
     * The class constructor.
     * @param dimension the puzzle width and height
     */
    public PuzzleModel(int dimension) {
        maxWidth = dimension + 1;
        maxHeight = dimension + 1;
        matPuzzle = new int[maxWidth][maxHeight];
        initPuzzle();
    }

    /**
     * Returns the puzzle matrix.
     * @return the puzzle matrix
     */
    public int[][] getMatPuzzle() {
        return matPuzzle;
    }

    /**
     * Gets the array of the numbers used for the game.
     * @return numbers
     */
    public int[] getNumbers() {
        return numbers;
    }

    /**
     * Gets the number of the used numbers for puzzle.
     * @return countNumbers
     */
    public int getCountNumbers() {
        return countNumbers;
    }

    /**
     * Sets the number of the used numbers for puzzle.
     * @param cNumbers - an int value
     */
    public void setCountNumbers(int cNumbers) {
        this.countNumbers = cNumbers;
    }

    /**
     * Returns the number of seconds since the game has started.
     * @return the number of seconds since the game has started
     */
    public int getCurrentSeconds() {
        return currentSeconds;
    }

    /**
     * Sets the number of seconds since the game has started.
     * @param cSeconds the number of seconds since the game has started
     */
    public void setCurrentSeconds(int cSeconds) {
        this.currentSeconds = cSeconds;
    }

    /**
     * Returns the number of minutes since the game has started.
     * @return the number of minutes since the game has started
     */
    public int getCurrentMinutes() {
        return currentMinutes;
    }

    /**
     * Sets the number of minutes since the game has started.
     * @param cMinutes the number of minutes since the game has started
     */
    public void setCurrentMinutes(int cMinutes) {
        this.currentMinutes = cMinutes;
    }

    /**
     * Returns the number of moves since the game has started.
     * @return the number of moves since the game has started
     */
    public int getCountMoves() {
        return countMoves;
    }

    /**
     * Sets the number of moves since the game has started.
     * @param cMoves the number of moves since the game has started
     */
    public void setCountMoves(int cMoves) {
        this.countMoves = cMoves;
    }

    /**
     * Returns the size of a piece.
     * @return the size of a piece
     */
    public int getPieceSize() {
        return pieceSize;
    }

    /**
     * Gets the maximum allowed width of the puzzle game drawing.
     * @return the maximum allowed width of the puzzle game drawing
     */
    public int getMaxWidth() {
        return maxWidth;
    }

    /**
     * Gets the maximum allowed height of the puzzle game drawing.
     * @return the maximum allowed height of the puzzle game drawing
     */
    public int getMaxHeight() {
        return maxHeight;
    }

    /**
     * Gets the X current position in the puzzle game.
     * @return the X current position in the puzzle game
     */
    public int getCurrentX() {
        return currentX;
    }

    /**
     * Sets the X current position in the puzzle game.
     * @param xCurrent the X current position in the puzzle game
     */
    public void setCurrentX(int xCurrent) {
        this.currentX = xCurrent;
    }

    /**
     * Gets the Y current position in the puzzle game.
     * @return the Y current position in the puzzle game
     */
    public int getCurrentY() {
        return currentY;
    }

    /**
     * Sets the Y current position in the puzzle game.
     * @param yCurrent the Y current position in the puzzle game
     */
    public void setCurrentY(int yCurrent) {
        this.currentY = yCurrent;
    }

    /**
     * Gets the start X position of drawing area for the puzzle table.
     * @return the start X position of drawing area for the puzzle table
     */
    public int getPuzzleX() {
        return puzzleX;
    }

    /**
     * Sets start X position of drawing area for the puzzle table.
     * @param xPuzzle start X position of drawing area for the puzzle table
     */
    public void setPuzzleX(int xPuzzle) {
        this.puzzleX = xPuzzle;
    }

    /**
     * Gets start Y position of drawing area for the puzzle table.
     * @return start Y position of drawing area for the puzzle table
     */
    public int getPuzzleY() {
        return puzzleY;
    }

    /**
     * Sets start Y position of drawing area for the puzzle table.
     * @param yPuzzle start Y position of drawing area for the puzzle table
     */
    public void setPuzzleY(int yPuzzle) {
        this.puzzleY = yPuzzle;
    }

    /**
     * Returns the status bar height.
     * @return the status bar height
     */
    public int getStatusBarHeight() {
        return statusBarHeight;
    }

    /**
     * Returns <code>true</code> if the current game has a winner.
     * @return <code>true</code> if the current game has a winner
     */
    public boolean isWinner() {
        return winner;
    }

    /**
     * Sets <code>true</code> if the current game has a winner.
     * @param winnerValue <code>true</code> if the current game has a winner
     */
    public void setWinner(boolean winnerValue) {
        this.winner = winnerValue;
    }

    /**
     * Initializes the puzzle table.
     */
    private void initPuzzle() {
        countNumbers = 0;
        numbers = new int[maxHeight * maxWidth];
        for (int j = 0; j < maxHeight; j++) {
            for (int i = 0; i < maxWidth; i++) {
                matPuzzle[j][i] = 0;
                countNumbers++;
                numbers[countNumbers - 1] = countNumbers;
            }
        }
        currentY = maxWidth - 1;
        currentX = maxHeight - 1;
    }

}
