/**
 * Copyright 2006 Ramona Dromereschi
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/
package com.puzzle.game.impl;

import com.puzzle.game.NewGameListener;

import javax.swing.JComponent;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * Created by IntelliJ IDEA.
 * <p/>
 * User: Ramona Dromereschi
 * <p/>
 * Date: Oct 25, 2006
 * Time: 1:20:40 PM
 * <p/>
 * The puzzle view.
 */
public class PuzzleView extends JComponent implements KeyListener {
    private boolean painted = false;
    /** The puzzle model. */
    private PuzzleModel model;
    /** The puzzle controller. */
    private PuzzleController controller;
    /** The list of new game event observers. */
    private List<NewGameListener> listeners = new ArrayList<NewGameListener>();

    /**
     * The class constructor.
     * @param c the puzzle controller
     * @param m the puzzle model
     */
    public PuzzleView(PuzzleController c, PuzzleModel m) {
        controller = c;
        model = m;
        Thread timerThread = new Thread(new TimerThread(this, model));
        timerThread.start();
    }

    /**
     * The paint method.
     * @param g the Graphics object
     */
    @Override
    public void paint(Graphics g) {
        controller.updateSize();
        Graphics2D g2 = (Graphics2D) g;
        g2.setColor(Color.GRAY);
        paintPuzzle(g2);
        paintStatusBar(g2);
    }

    /**
     * Paints only the puzzle area.
     * @param g2 the Graphics2D object
     */
    private void paintPuzzle(Graphics2D g2) {
        g2.setColor(Color.WHITE);
        g2.fillRect(0, 0, getWidth(), getHeight());
        Font f = g2.getFont();
        for (int j = 0; j < model.getMaxHeight(); j++) {
            for (int i = 0; i < model.getMaxWidth(); i++) {
                if ((i != model.getCurrentX()) || (j != model.getCurrentY())) {
                    if (!painted) {
                        model.getMatPuzzle()[j][i] = controller.getRandomNumber();
                    }
                    g2.setColor(Color.GRAY);
                    g2.drawRect(model.getPuzzleY() + i * model.getPieceSize(),
                            model.getPuzzleX() + j * model.getPieceSize(),
                            model.getPieceSize(), model.getPieceSize());
                    int r = Math.abs(((model.getMatPuzzle()[j][i] - 1) / model.getMaxWidth()) * 20);
                    int g = Math.abs(((model.getMatPuzzle()[j][i] - 1) / model.getMaxWidth()) * 25);
                    int b = Math.abs(((model.getMatPuzzle()[j][i] - 1) / model.getMaxWidth()) * 30);
                    g2.setColor(new Color(255 - r, 255 - g, 255 - b));
                    g2.fillRect(model.getPuzzleY() + i * model.getPieceSize() + 1,
                            model.getPuzzleX() + j * model.getPieceSize() + 1,
                            model.getPieceSize() - 1, model.getPieceSize() - 1);
                    g2.setColor(Color.BLUE);
                    String str = Integer.toString(model.getMatPuzzle()[j][i]);
                    FontMetrics m = g2.getFontMetrics(f);
                    g2.drawString(str,
                            model.getPuzzleY() + i * model.getPieceSize() + m.stringWidth(str) / 2,
                            model.getPuzzleX() + j * model.getPieceSize() + m.getHeight());
                }
            }
        }
    }

    /**
     * Paints only the status bar area.
     * @param g2 the Graphics2D object
     */
    private void paintStatusBar(Graphics2D g2) {
        g2.setColor(new Color(240, 230, 170));
        int statusY = getHeight() - model.getStatusBarHeight();
        g2.fillRect(0, statusY, getWidth(), model.getStatusBarHeight());
        Font f = g2.getFont();
        g2.setColor(Color.GRAY);
        String str = "Number of moves " + model.getCountMoves();
        FontMetrics m = g2.getFontMetrics(f);
        int statusTextY = statusY + f.getSize() + 2;
        g2.drawString(str, 2 + m.stringWidth(str) / 2, statusTextY);
        if (painted) {
            if (controller.isWinner()) {
                if (model.getCountMoves() > 0) {
                    g2.setColor(Color.BLUE);
                    g2.drawString("Congratulations!", 200 + m.stringWidth(str) / 2, statusTextY);
                    g2.setColor(Color.GRAY);
                } else {
                    painted = false;
                    return;
                }
            }
        }
        str = "Time " + model.getCurrentMinutes() + ":" + model.getCurrentSeconds();
        g2.drawString(str, 140 + m.stringWidth(str) / 2, statusTextY);
        g2.drawString("Press SPACE for new game!", 320 + m.stringWidth(str) / 2, statusTextY);
        painted = true;
    }

    public void keyTyped(KeyEvent keyEvent) {
    }

    public void keyPressed(KeyEvent keyEvent) {
    }

    public void keyReleased(KeyEvent keyEvent) {
        if (keyEvent.getKeyCode() == KeyEvent.VK_SPACE) {
            notifyNewGameObservers();
        }
        controller.updatePuzzle(keyEvent.getKeyCode());
    }

    public synchronized void addNewGameListener(NewGameListener listener) {
        listeners.add(listener);
    }

    public synchronized void removeNewGameListener(NewGameListener listener) {
        listeners.remove(listener);
    }

    private synchronized void notifyNewGameObservers() {
        for (NewGameListener listener : listeners) {
            listener.newGameRequest();
        }
    }
}